--

package Bitmap_Format_Rv1 is

   type Signed_8_Bit_Integer is new Integer range -2**7 .. 2**7 - 1 with
        Size => 8;

   type Unsigned_8_Bit_Integer is new Integer range 0 .. 2**8 - 1 with
        Size => 8;

   type Signed_16_Bit_Integer is new Integer range -2**15 .. 2**15 - 1 with
        Size => 16;

   type Unsigned_16_Bit_Integer is new Integer range 0 .. 2**16 - 1 with
        Size => 16;

   type Signed_32_Bit_Integer is range -2**31 .. 2**31 - 1 with
        Size => 32;

   type Unsigned_32_Bit_Integer is range 0 .. 2**32 - 1 with
        Size => 32;

      -- -- --

   subtype Byte_Type is Unsigned_8_Bit_Integer;

   Null_Byte : constant Byte_Type := 0;

   subtype Uint16_T is Unsigned_16_Bit_Integer;
   subtype Uint32_T is Unsigned_32_Bit_Integer;

   subtype Int16_T is Signed_16_Bit_Integer;
   subtype Int32_T is Signed_32_Bit_Integer;

   -- -- --

   subtype Colour_Value_Type is Byte_Type;

   type Pixel_Colour_Type is record
      B : Colour_Value_Type := Null_Byte;
      G : Colour_Value_Type := Null_Byte;
      R : Colour_Value_Type := Null_Byte;
   end record with
      Size => 24;

   for Pixel_Colour_Type use record
      B at 0 range  0 ..  7;
      G at 0 range  8 .. 15;
      R at 0 range 16 .. 23;
   end record;

   -- -- --

   subtype Index_Type is Int32_T range 1 .. Int32_T'Last;

   subtype Length_Type is Int32_T range 1 .. Int32_T'Last;

   type Row_Colours_Array_Type is array (Positive range <>) of Pixel_Colour_Type with
        Component_Size => 3 * 8;

      --
      --  ### Konstruktion eines passgenauen Bitmap-Typen Klappt nicht ###
      --

      -- -- --
      -- -- --

   type Bitmap_Header_Type is record
      --
      -- Bitmap_File_Header 14 Byte

      Bf_Type : Uint16_T := 16#4D42#;
      --
      -- "BM", 10#19778#

      Bf_Size : Uint32_T;
      --
      --  Gre der BMP-Datei in Byte (unzuverlssig)

      Bf_Reserved : Uint32_T := 0;
      --
      --  Reserviert, von der Software abhngig, standardmig 0

      Bf_Offbits : Uint32_T := 54;
      --
      --  Offset der Bilddaten in Byte vom Beginn der Datei an, bei Echtfarben fast immer 54 (manche
      --  Software ignoriert diese Angabe daher fehlerhafterweise)

      -- -- --
      -- -- --

      --  Bitmap_Infomation_Header 40 Byte

      Bi_Size : Uint32_T := 40;
      --
      --  40 (Gre der BITMAPINFOHEADER-Struktur in Byte)

      Bi_Width : Int32_T;
      --
      -- Breite der Bitmap in Pixel

      Bi_Height : Int32_T;
      --
      --  Der Betrag gibt die Hhe der Bitmap in Pixel an Ist der Wert positiv, so ist die Bitmap
      --  eine sogenannte "bottom-up"-Bitmap (die Bilddaten beginnen mit der untersten und enden mit
      --  der obersten Bildzeile). Dies ist die gebruchlichste Variante Ist der Wert negativ, so
      --  ist die Bitmap eine "top-down"-Bitmap (die Bilddaten beginnen mit der obersten und enden
      --  mit der untersten Bildzeile)

      Bi_Planes : Uint16_T := 1;
      --
      --  1 (Stand in einigen lteren Formaten wie PCX fr die Anzahl der Farbebenen, wird aber
      --  fr BMP nicht verwendet)

      Bi_Bit_Count : Uint16_T := 24;
      --
      --  Gibt die Farbtiefe der Bitmap in bpp an; Muss einer der folgenden Werte sein : 1, 4, 8,
      --  16, 24 oder 32. Bei 1, 4 und 8 bpp sind die Farben indiziert

      Bi_Compression : Uint32_T := 0;
      --
      --  Einer der folgenden Werte: 0 (BI_RGB): Bilddaten sind unkomprimiert. etc ...

      Bi_Size_Image : Uint32_T := 0;
      --
      --  Wenn bi_Compression=BI_RGB: Entweder 0 oder die Gre der Bilddaten in Byte

      Bi_X_Pels_Per_Meter : Int32_T := 0;
      --
      --  Horizontale Auflsung des Zielausgabegertes in Pixel pro Meter; Wird aber fr BMP -
      --  Dateien meistens auf 0 gesetzt

      Bi_Y_Pels_Per_Meter : Int32_T := 0;
      --
      --  Vertikale Auflsung des Zielausgabegertes in Pixel pro Meter; wird aber fr BMP -
      --  Dateien meistens auf 0 gesetzt

      Bi_Clr_Used : Uint32_T := 0;
      --
      --  ... Ansonsten: Die Anzahl der Eintrge der Farbtabelle (0=keine Farbtabelle)

      Bi_Clr_Important : Uint32_T := 0;
      --
      --  ... Ansonsten: Wenn eine Farbtabelle vorhanden ist und diese smtliche im Bild
      --  verwendeten Farben enthlt : deren Anzahl ... Ansonsten: 0

   end record;

   for Bitmap_Header_Type use record
      Bf_Type             at  0 range 0 .. 15;
      Bf_Size             at  2 range 0 .. 31;
      Bf_Reserved         at  6 range 0 .. 31;
      Bf_Offbits          at 10 range 0 .. 31;
      Bi_Size             at 14 range 0 .. 31;
      Bi_Width            at 18 range 0 .. 31;
      Bi_Height           at 22 range 0 .. 31;
      Bi_Planes           at 26 range 0 .. 15;
      Bi_Bit_Count        at 28 range 0 .. 15;
      Bi_Compression      at 30 range 0 .. 31;
      Bi_Size_Image       at 34 range 0 .. 31;
      Bi_X_Pels_Per_Meter at 38 range 0 .. 31;
      Bi_Y_Pels_Per_Meter at 42 range 0 .. 31;
      Bi_Clr_Used         at 46 range 0 .. 31;
      Bi_Clr_Important    at 50 range 0 .. 31;
   end record;

   -- -- --

   -- Bilddaten

   --  Die Bilddaten beginnen am Offset bfOffBits. Die Gre der Bilddaten betrgt
   --  nherungsweise (gilt nur fr durch 4 teilbare Bildbreiten) biWidthbiHeightbiBitCount
   --  / 8 wenn biCompression = BI_RGB, ansonsten biSizeImage Die Bilddaten werden Zeile fr Zeile
   --  gespeichert. Wenn biHeight positiv ist, beginnen die Bilddaten mit der letzten und enden mit
   --  der ersten Bildzeile, ansonsten ist es umgekehrt. Die Lnge jeder Zeile ist ein Vielfaches
   --  von 4 Bytes, falls erforderlich, wird mit Nullbytes aufgefllt
   --
   --  24 bpp: Die Daten jedes Pixels bestehen aus jeweils einem Byte fr den Blau - , Grn - und
   --  Rot - Kanal (in dieser Reihenfolge!).

end Bitmap_Format_Rv1;
