--

with Printer;
use Printer;

with Ada.Exceptions;

with Ada.Task_Identification;

separate (Completion_Pool_Executors)
task body Worker_Task_Type is
   use Ada;
   use type Task_Identification.Task_Id;

   My_Callable : My_Callables.Any_Callable_Access_Type;

   My_Result : My_Executors.Result_Type;
   My_Params : My_Executors.Parameter_Type;

   My_Result_Future : My_Futures.Any_Future_Access_Type;

   My_Executor : Any_Completion_Pool_Executor_Access_Type;

   My_Id : Positive;
   Found : Boolean := False;

begin
   My_Executor := My_Task_Attributes.Value;
   --
   --  Attrubutes of the current task

   Search_For_My_Id :
   for Ix in My_Executor.Pool'Range loop

      if (My_Executor.Pool (Ix)'Identity = Task_Identification.Current_Task) then
         Found := True;
         My_Id := Ix;

         exit Search_For_My_Id;
      end if;

   end loop Search_For_My_Id;

   -- -- --

   if not Found then
      raise Program_Error;
   end if;

   -- -- --

   Do_My_Work :
   loop

      My_Executor.Controller.Put (My_Id);
      --
      -- I am ready to do the work

      select

         accept Fork
           (This   : in My_Callables.Any_Callable_Access_Type;
            Params : in My_Executors.Parameter_Type;
            Result : in My_Futures.Any_Future_Access_Type) do
            --
            My_Callable      := This;
            My_Params        := Params;
            My_Result_Future := Result;
         end Fork;

      or
         terminate;
      end select;

      My_Result := My_Callable.Call (My_Params);
      --
      --  Execute the working routine

      My_Result_Future.Set (My_Result);

      My_Executor.Future_Buffer.Put (My_Result_Future);

   end loop Do_My_Work;

exception
   when Event : others =>
      Printer_Task.Print ("#! Completion_Pool_Executors - Worker_Task_Type : EXCEPTION");
      Printer_Task.Print (Exceptions.Exception_Information (Event));

      raise;

end Worker_Task_Type;
