--

with Ada.Finalization;

with System.Multiprocessors;

with Executors.Completion_Services;
with Buffers.Mailboxes;

generic
   with package My_Executors is new Executors (<>);
   use My_Executors;

   with package My_Completion_Services is new My_Executors.Completion_Services (<>);
   use My_Completion_Services;

package Completion_Pool_Executors is
   use Ada;

   type Completion_Pool_Executor_Type (Pool_Size : Positive) is
     limited new Finalization.Limited_Controlled and Completion_Service_Type with private;

   type Any_Completion_Pool_Executor_Access_Type is access all Completion_Pool_Executor_Type'Class;

   overriding procedure Submit
     (Exec     : in out Completion_Pool_Executor_Type;
      Callable : in     My_Callables.Any_Callable_Access_Type;
      Params   : in     My_Executors.Parameter_Type;
      Future   : in     My_Futures.Any_Future_Access_Type);

   overriding procedure Take
     (Exec   : in out Completion_Pool_Executor_Type;
      Future :    out My_Futures.Any_Future_Access_Type);

private

   Task_Nb : Integer := -1 with
      Atomic;

   My_Number_Of_Cpus : Natural := Natural (System.Multiprocessors.Number_Of_CPUs) with
      Atomic;

   function Set_Cpu return System.Multiprocessors.CPU_Range;

   -- -- --

   task type Worker_Task_Type is
      pragma Cpu (Set_Cpu);

      entry Fork
        (This   : in My_Callables.Any_Callable_Access_Type;
         Params : in My_Executors.Parameter_Type;
         Result : in My_Futures.Any_Future_Access_Type);
   end Worker_Task_Type;

   type My_Worker_Pool_Task_Array_Type is array (Positive range <>) of Worker_Task_Type;

   -- -- --

   package My_Futures_Buffers is new Buffers (My_Futures.Any_Future_Access_Type);
   package My_Future_Maiboxes is new My_Futures_Buffers.Mailboxes;

   package My_Task_Buffers is new Buffers (Positive);
   package My_Task_Mailboxes is new My_Task_Buffers.Mailboxes;

   -- -- --

   type Completion_Pool_Executor_Type
     (Pool_Size : Positive) is limited new Finalization.Limited_Controlled and
     Completion_Service_Type with
   record
      Controller    : My_Task_Mailboxes.Mailbox_Type (Pool_Size);
      Pool          : My_Worker_Pool_Task_Array_Type (Positive'First .. Pool_Size);
      Future_Buffer : My_Future_Maiboxes.Mailbox_Type (Pool_Size);
   end record;

   -- -- --
   -- -- --

   procedure Initialize (Excutor : in out Completion_Pool_Executor_Type);
   --
   --  Used with Ada.Finalization, does the initialization after creating an Executor

end Completion_Pool_Executors;
